/* evtgen configuration parser.
   Copyright (C) 2003 Julien Olivain and LSV, CNRS UMR 8643 & ENS Cachan.

   This file is part of evtgen.

   evtgen is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   evtgen is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with evtgen; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

%{

#include <stdlib.h>
#include <stdio.h>

#include "evtgen.h"

extern int parserlineno_g;
extern char *parsertext;

%}

%union {
  float prob;
  char *str;
  symbol_token_t sym;
  text_token_t *tok;
  value_t *event;
  random_var_t *randvar;
}

%token <sym> SYM VAR BUILTIN_VAR
%token <str> STR
%token <prob> PROB

%type <prob> prob
%type <tok> event_decl_term event_decl
%type <event> event_list event
%type <randvar> randvar_list randvar

%%

globaldef:
  /* empty */
  { printf("empty configuration file\n"); }
| randvar_list
  { build_randvar_list($1); }
;

randvar_list:
  randvar_list randvar
    { $2->next = $1; $$ = $2; }
| randvar
    { $$ = $1; }
;

randvar:
  SYM '{' event_list '}'
    { $$ = build_randvar($1.name, $1.line, $3); }
;

event_list:
  event_list event
    { $2->next = $1; $$ = $2; }
| event
    { $$ = $1; }
;

event:
  prob ':' event_decl ';'
    { $$ = build_event($1, $3); }
;

prob:
  PROB
    { $$ = $1; }
| '-'
    { $$ = -1.0f; }
;

event_decl:
  event_decl event_decl_term
    { $2->generic.next = $1; $$ = $2; }
| event_decl_term
    { $$ = $1; }
;

event_decl_term:
  STR
    { $$ = build_string_token($1); }
| BUILTIN_VAR
    { $$ = build_builtin_var_token($1.name); }
| VAR
    { $$ = build_randvar_token($1.name); }
;

%%

void
parsererror(char *s)
{
  DPRINTF( ("%s on line %i at '%s'.\n", s, parserlineno_g, parsertext) );
  DPRINTF( ("have to clear current compiler context (XXX: ToDo)\n") );
}
